; AKern CPU Interrupt Setup
; Includes routines and IDT
[bits 32]

idt:
times 256 dq 0
idt_end:

idtr:
    dw idt_end - idt - 1
    dd idt

isr_setup:
    ; Add in the Exceptions
    mov edx, 0x0
    mov eax, exc_div
    call set_isr
    mov edx, 0xD
    mov eax, exc_general_protection_fault
    call set_isr
    mov edx, 0xE
    mov eax, exc_page_fault
    call set_isr
    ; Add in the ISRs
    mov edx, 0x12; setstat - to test if interrupts are working
    mov eax, isr_setstat
    call set_isr
    mov edx, 0x13; userwrite - to see if interrupts work in userland
    mov eax, isr_userwrite
    call set_isr
    ; Add in the IRQs
    mov edx, 0x21; keypress handler
    mov eax, irq_kb
    call set_isr

    ret

idt_setup:
    ; Filling the Interrupt Descriptor Table with a dummy interrupt
    call fill_idt
    ; modifying the intel reserved interrupt entries
    mov edx, 1
    call set_pres
    mov edx, 14
    call set_pres
    ret

set_isr: ; ecx is the counter, edx contains the target, ebx holds the address
    pusha
    mov ecx, 0
    mov ebx, idt
    mov dword [israddr], eax
    mov eax, israddr
    jmp .loop
.setdata:
    push eax
    mov ax, [eax]
    mov word [ebx], ax
    pop eax
    add ebx, 6
    add eax, 2
    push eax
    mov ax, [eax]
    mov word [ebx], ax
    pop eax
    jmp .end
.loop:
    cmp ecx, edx
    je .setdata
    add ebx, 8
    inc ecx
    jmp .loop
.end:
    popa
    ret

set_pres: ; ecx is the counter, edx contains the target, ebx holds the address
    pusha
    mov ecx, 0
    mov ebx, idt
    jmp .loop
.setdata:
    add ebx, 4
    mov word [ebx], 0xE00
    jmp .end
.loop:
    cmp ecx, edx
    je .setdata
    add ebx, 8
    inc ecx
    jmp .loop
.end:
    popa
    ret

fill_idt:
    pusha
    mov edx, idt_end
    mov ecx, idt
    mov dword [israddr], isr_dummy
    mov ebx, israddr
.loop:
    cmp edx, ecx
    je .end

    mov ax, [ebx]
    mov word [ecx], ax ;3693

    add ecx, 2
    mov word [ecx], CSEG_1;0x10 ;1000

    add ecx, 2
    mov word [ecx], 0x8E00 ; 008E

    add ecx, 2
    add ebx, 2
    mov ax, [ebx]
    mov word [ecx], ax ;0000

    add ecx, 2

    sub ebx, 2
    jmp .loop
.end:
    popa
    ret

disp_idt: ; function to display the idt
    mov ecx, idt
.loop:
    mov al, [ecx]
    call byteout
    inc ecx
    cmp ecx, idt_end
    jne .loop
    ret

israddr: dd 0

%include "isr.asm"
%include "irq.asm"
