; AKern Disk I/O routines
; Right now, only compatible with IDE disks
[bits 32]

diskout: ; Store in any part of the disk
; edx = source in memory
; ah = drive and head port
; bl = sector count port
; bh = sector number port
; cl = cylinder low port
; ch = cylinder high port
push edx
	mov     dx,1f6h         ;Drive and head port
	mov     al,ah
	out     dx,al

	mov     dx,1f2h         ;Sector count port
	mov     al,bl
	out     dx,al

	mov     dx,1f3h         ;Sector number port
	mov     al,bh
	out     dx,al

	mov     dx,1f4h         ;Cylinder low port
	mov     al,cl
	out     dx,al

	mov     dx,1f5h         ;Cylinder high port
	mov     al,ch
	out     dx,al

	mov     dx,1f7h         ;Command port
	mov     al,30h          ;Read with retry.
	out     dx,al
.buff_loop:
	in      al,dx
	test    al,8
	jz      .buff_loop

	pop edx
	mov     ecx,512/2        ;One sector /2
	mov     edi,edx
	mov     edx,1f0h         ;Data port - data comes in and out of here.
	rep     outsw

	ret


diskin: ; Load any part of the disk
; edx = destination in memory
; ah = drive and head port
; bl = sector count port
; bh = sector number port
; cl = cylinder low port
; ch = cylinder high port
push edx
	mov     dx,1f6h         ;Drive and head port
	mov     al,ah
	out     dx,al

	mov     dx,1f2h         ;Sector count port
	mov     al,bl
	out     dx,al

	mov     dx,1f3h         ;Sector number port
	mov     al,bh
	out     dx,al

	mov     dx,1f4h         ;Cylinder low port
	mov     al,cl
	out     dx,al

	mov     dx,1f5h         ;Cylinder high port
	mov     al,ch
	out     dx,al

	mov     dx,1f7h         ;Command port
	mov     al,20h          ;Read with retry.
	out     dx,al
.buff_loop:
	in      al,dx
	test    al,8
	jz      .buff_loop

	pop edx
	mov     ecx,512/2        ;One sector /2
	mov     edi,edx
	mov     edx,1f0h         ;Data port - data comes in and out of here.
	rep     insw

	ret



;;
;;	The Following are Only for Reference
;;

loadt: ; Load a location in the bootsector
	mov     dx,1f6h         ;Drive and head port
	mov     al,0a0h         ;Drive 0, head 0
	out     dx,al

	mov     dx,1f2h         ;Sector count port
	mov     al,1            ;Read one sector
	out     dx,al

	mov     dx,1f3h         ;Sector number port
	mov     al,1            ;Read which sector?
	out     dx,al

	mov     dx,1f4h         ;Cylinder low port
	mov     al,0            ;Cylinder 0
	out     dx,al

	mov     dx,1f5h         ;Cylinder high port
	mov     al,0            ;The rest of the cylinder 0
	out     dx,al

	mov     dx,1f7h         ;Command port
	mov     al,20h          ;Read with retry.
	out     dx,al
.buff_loop:
	in      al,dx
	test    al,8            ;This means the sector buffer requires
				;servicing.
	jz      .buff_loop      ;Don't continue until the sector buffer
				;is ready.

	mov     ecx,512/2        ;One sector /2
	mov     edi,0
	mov     edx,1f0h         ;Data port - data comes in and out of here.
	rep     insw

	ret


loads: ; Load a sector from after the Kernel
	mov     dx,1f6h         ;Drive and head port
	mov     al,0a0h         ;Drive 0, head 0
	out     dx,al

	mov     dx,1f2h         ;Sector count port
	mov     al,1            ;Read one sector
	out     dx,al

	mov     dx,1f3h         ;Sector number port
	mov     al,20           ;Read which sector? (boot sector + kernel + padding)
	out     dx,al

	mov     dx,1f4h         ;Cylinder low port
	mov     al,0            ;Cylinder 0
	out     dx,al

	mov     dx,1f5h         ;Cylinder high port
	mov     al,0            ;The rest of the cylinder 0
	out     dx,al

	mov     dx,1f7h         ;Command port
	mov     al,20h          ;Read with retry.
	out     dx,al
.buff_loop:
	in      al,dx
	test    al,8            ;This means the sector buffer requires
				;servicing.
	jz      .buff_loop      ;Don't continue until the sector buffer
				;is ready.

	mov     ecx,512/2        ;One sector /2
	mov     edi,0x200000
	mov     edx,1f0h         ;Data port - data comes in and out of here.
	rep     insw

	ret
