; AKern Display Routines & Data
; Handle overflow, so are suitable for syscalls
[bits 32]
    ;;; Routines

; Set the row for text to display
; and add to it
addrow:
    mov ecx, [cur_row]
    add ecx,eax
    mov [cur_row], ecx
    call setrow
    ret

; Set the row for text to display
; and increment automatically
incrow:
    mov ecx, [cur_row]
    inc ecx
    mov [cur_row], ecx
    call setrow
    ret

; Set the row for text to display.
; ecx=row to set
setrow:
    mov ebx,0xB8000
.loop:
    cmp ecx,0
    jz .end
    add ebx,160
    cmp ebx, 0xB8fff
    jg .overflow
    dec ecx
    jmp .loop
.overflow:
    mov ebx, 0xB8000
    dec ecx
    jmp .loop
.end:
    mov [curchar], ebx
    ret

; cleardisp (no parameters)
cleardisp:
    mov ecx, 0xB8000
.loop:
    mov word [ecx], 0
    add ecx, 2
    cmp ecx, 0xB9000
    jne .loop
    ret

; strout (white on black)
; eax=string pointer
strout:
mov ecx, [curchar]
dec eax
mov bh, 0x0f
.loop:
    inc eax         ; increment string pointer
    mov bl, [eax]   ; get character from string
    cmp bl, 0       ; check for null terminator
    je .end
    mov [ecx], bx   ; Move character into video memory
    add ecx,2       ; increment VRAM pointer
    cmp ecx, 0xB9000    ; check for overflow
    je .overflow
    jmp .loop
.overflow:
    mov ecx, 0xB8000
    jmp .loop
.end:
    mov [curchar],ecx
    ret

; strout (any color combination)
; eax=string pointer, bl=color combo
strout_color:
mov ecx, [curchar]
dec eax
mov bh, bl
.loop:
    inc eax         ; increment string pointer
    mov bl, [eax]   ; get character from string
    cmp bl, 0       ; check for null terminator
    je .end
    mov [ecx], bx   ; Move character into video memory
    add ecx,2       ; increment VRAM pointer
    cmp ecx, 0xB9000    ; check for overflow
    je .overflow
    jmp .loop
.overflow:
    mov ecx, 0xB8000
    jmp .loop
.end:
    mov [curchar],ecx
    ret

; charout (any color combination)
; ax=full color and character code
charout:
    mov ecx, [curchar]
    mov [ecx], ax
    add ecx, 2
    cmp ecx, 0xB9000    ; check for overflow
    je .overflow
    mov [curchar],ecx
    jmp .end
.overflow:
    mov ecx, 0xB8000
.end:
    ret

byteout:
    pusha
    mov dword [hexstrout], 0
    mov dl, al
    mov ecx, hexstrout
    call hexstr
    mov eax, hexstrout
    call strout
    popa
    ret

bytes_out: ; doesn't handle overflow
.loop:
    cmp ecx, 0
    jz .end
    mov al, [eax]
    call byteout
    inc eax
    dec ecx
    jmp .loop
.end:
    ret

    ; Data
curchar: dd 0xB8000
