; Useful routines and other stuff that AKern uses
[bits 32]

;;
;; Routines
;;

; A tss descriptor for testing
;tss_desc:
;    dq 0
;tss_desc_end:

; An Actual TSS
tss:
    dd 0        ; previous TSS
    dd 0x10ffff ; ESP0
    dd 0x10 ; SS0
    ;dw 0
    times 0x17 dd 0
tss_end:

    IS_CPUID_SUPPORTED:
pushfd
pushfd
xor dword [esp],0x00200000
popfd
pushfd
pop eax
xor eax, [esp]
popfd
add eax, 0x00200000
ret

; cmp_str - compare two strings
; eax = string A, ebx = string B
; edx=0: not equal
cmp_str:
.loop:
    mov dl, [eax]
    mov dh, [ebx]
    cmp dl, dh
    jne .neq
    cmp dl, 0
    jz .end
    inc eax
    inc ebx
    jmp .loop
.neq:
    mov edx, 0
    ret
.end:
    mov edx, 1
    ret

hexstr: ; turn byte into hexadecimal string
;; dl contains the byte, ecx contains the string pointer
    push ebx

    mov   ebx, HexConvTABLE
    mov   ax, dx

    mov   ah, al            ;make al and ah equal so we can isolate each half of the byte
    shr   ah, 4             ;ah now has the high "digit"
    and   al, 0x0F          ;al now has the low "digit"
    xlat                    ;lookup al's contents in our table
    xchg  ah, al            ;flip around the bytes so now we can get the higher
    xlat                    ;look up what we just flipped

    mov   ebx, ecx
    ;xchg  ah, al
    mov   [ebx], ax          ;append the new hex to the string of bytes

    pop ebx
    ret

tss_set_stack:
    pop esi
    mov eax, tss
    add eax, 4
    mov [eax], esp
    push esi
    ret

tss_desc_base: dd 0
tss_desc_limit: dd 0

set_tss_desc_updated:
    mov eax, tss_desc   ; Load the address of the TSS descriptor
    mov ebx, tss        ; Load the base address of the TSS
    mov ecx, tss_end    ; Load the limit of the TSS
    sub ecx, ebx

    ; Fill the TSS descriptor
    mov word [eax], cx  ; Set the limit (15:0)
    shr ecx, 16
    mov [eax+6], bl     ; Set the base (15:0)
    shr ebx, 16
    mov [eax+7], bh     ; Set the base (23:16)
    mov [eax+8], cl     ; Set the limit (19:16) and base (31:24)
    mov byte [eax+9], 0x89   ; Set the type (0x89 for available 32-bit TSS) and present bit
    mov byte [eax+10], 0     ; Set the reserved bits
    mov [eax+11], bh    ; Set the base (31:24)
    ret


set_tss_desc: ; Sets the TSS descriptor
; ecx   = base
; edx   = limit
; Finally got this thing working
;   after hours of researching
;   and thinking.

; STEPS OF SETTING THE TSS DESCRIPTOR:
;   Set WORD to the first 16 bits of Limit
;   Set WORD to the first 16 bits of Base
;   Set BYTE to the next byte of Base
;   Set BYTE to 0b10010001
;   Set BYTE to final hex digit of Limit + 0000
;   Set BYTE to final byte of Base
; STEPS OF RETRIEVING PORTIONS OF ADDRESSES:
;   For getting LIMIT[0..15]
;       Start at BIT 0 of LIMIT.
;       shl limit, 0
;       GET word limit
;   For getting BASE[0..15]
;       Start at BIT 0 of BASE.
;       shl base, 0
;       GET word base
;   For getting BASE[16..23]
;       Start at BIT 16 of BASE.
;       shl base, 16
;       GET byte base
;   For getting LIMIT[16..19]
;       Start at BIT 16 of LIMIT.
;       shl limit, 16
;       AL = GET byte limit
;       and al, 0b11110000
;   For getting BASE[24..31]
;       Start at BIT 24 of BASE.
;       shl base, 24
;       GET byte base
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;mov [tss_desc_base], ecx
    ;mov [tss_desc_limit], edx
    ;mov ecx, tss_desc_base
    ;mov edx, tss_desc_limit
    ; EDX=LIMIT
    ; ECX=BASE

    mov eax, tss_desc   ; Load the address of the TSS descriptor
    mov ebx, tss        ; Load the base address of the TSS
    mov ecx, tss_end    ; Load the limit of the TSS
    sub ecx, ebx


    mov ebx, tss_desc
    ;---------------------
    ;       LIMIT [15..0];
    mov word [ebx], dx
    ;---------------------
    ;---------------------
    ;       BASE [15..0] ;
    add ebx, 2
    mov word [ebx], cx
    ;---------------------
    ;---------------------
    ;       BASE [23..16];
    add ebx, 2
    shr ecx, 16
    mov byte [ebx], cl
    ;---------------------
    ;---------------------
    ;       ...          ;
    inc ebx
    mov byte [ebx], 0x89 ; 0b10001001
    ;---------------------
    ;---------------------
    ;       LIMIT [19..16]
    inc ebx
    shr edx, 16
    mov al, dl
    ;and al, 0b11110000
    mov byte [ebx], al
    ;---------------------
    ;---------------------
    ;       BASE [31..24]
    inc ebx
    shr ecx, 7
    mov byte [ebx], cl
    ;  ----- DONE -----
    ret

disp_tss_desc: ; function to display the tss descriptor
    mov ebx, tss_desc
    mov esi, 0
.loop:
    mov al, [ebx]
    call byteout
    inc ebx
    inc esi
    cmp esi, 8
    jne .loop
    ret

enter_usermode: ; this has no flaws
    mov ax, (4 * 8) | 3 ; ring 3 data with bottom 2 bits set for ring 3
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax ; SS is handled by iret

	; set up the stack frame iret expects
	mov eax, esp
	push (4 * 8) | 3 ; data selector
	push eax ; current esp
	pushf ; eflags
	push (3 * 8) | 3 ; code selector (ring 3 code with bottom 2 bits set for ring 3)
	push 0x200000 ; instruction address to return to
	iret

;;
;; Data
;;

; For byte to hex-string conversion
HexConvTABLE:
    db "0123456789ABCDEF", 0
hexstrout: dq 0

; IBM PS/2 Keypress Translation Table.
; Only tested on an old IBM T42
KB_TABLE: db "`1234567890-=",0,0,"qwertyuiop[]",0,0,"asdfghjkl;'","`",0,"\","zxcvbnm,./",0,"I",0," ",0,"LM"
times 255 db 0
ST_TABLE: db "~!@#$%^&*()_+",0,0,"QWERTYUIOP{}",0,0,"ASDFGHJKL:",'"',"`",0,"|","ZXCVBNM<>?",0,"I",0," ",0,"LM"
times 255 db 0

; Other variables used by the Keypress Hanlder
keyp: dw 0
KSTAT: db 0
cur_sc: db 255
CURKEY: db 0
